/*! \file enc28j60.c \brief Microchip ENC28J60 Ethernet Interface Driver. */

#include <stdint.h>
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#include "enc28j60.h"
#include "common.h"
#include "spi.h"
#include "AT91SAM7S256.h"



//#define DSELECT() AT91F_PIO_SetOutput( AT91C_BASE_PIOA, AT91C_PA11_NPCS0 )
#define DSELECT() spiCsSelect(NOPCS)
//#define SELECT() AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, AT91C_PA11_NPCS0 )
#define SELECT() spiCsSelect (ETH_CS)


u08 Enc28j60Bank;
u16 NextPacketPtr;


u08 spi_send(u08 data) {
	AT91PS_SPI pSPI      = AT91C_BASE_SPI;
	while( !( pSPI->SPI_SR & AT91C_SPI_TDRE ) ); // transfer compl. wait
	pSPI->SPI_TDR = data;
	while( !( pSPI->SPI_SR & AT91C_SPI_RDRF ) ); // wait for char
	return pSPI->SPI_RDR ; // read RDR;
}

void delay(unsigned short us) 
{
    int a;
    for(a=0;a<100;a++) { nop(); }
} 

void nicSetMacAddress(u08* macaddr)
{
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, *macaddr++);
	enc28j60Write(MAADR4, *macaddr++);
	enc28j60Write(MAADR3, *macaddr++);
	enc28j60Write(MAADR2, *macaddr++);
	enc28j60Write(MAADR1, *macaddr++);
	enc28j60Write(MAADR0, *macaddr++);
}

u08 enc28j60ReadOp(u08 op, u08 address)
{

    u08 data;
    // release CS
	SELECT();
    
    spi_send( op | (address & ADDR_MASK));
    iprintf("Rop Send: %x\n",op | (address & ADDR_MASK));
    data = spi_send(0xff); // read data
	iprintf("Rop :%x\n",data); 

	if(address & 0x80) //MAC/MII REG
	{
	    data = spi_send(0xff); // read data
	}
	DSELECT(); //deselect CS Pin
	return data;
}

void enc28j60WriteOp(u08 op, u08 address, u08 data)
{
    // release CS
    SELECT();
    u08 dat;
    // issue write command
	dat = spi_send( op | (address & ADDR_MASK));
	iprintf("Wop op:%x --> %x\n",(op | (address & ADDR_MASK)),dat);
	dat = spi_send(data);
	iprintf("Wop dat:%x --> %x\n",data,dat);
	
    DSELECT(); //deselect CS Pin
}

void enc28j60ReadBuffer(u16 len, u16* data)
{
	// assert CS
    SELECT();
	// issue read command
	spi_send( ENC28J60_READ_BUF_MEM );
	while(len--)
	{
		// read data
		*data++ = spi_send(0xff);
	}	

	DSELECT(); //deselect CS Pin
}

void enc28j60WriteBuffer(u16 len, u16* data)
{
	// assert CS
	SELECT(); //CS auf Low
	// issue write command
	spi_send( ENC28J60_WRITE_BUF_MEM );

	while(len--)
	{
		// write data
		spi_send( *data++ );
	}	

	DSELECT(); //deselect CS Pin
}

void enc28j60SetBank(u08 address)
{
	// set the bank (if needed)
	if((address & BANK_MASK) != Enc28j60Bank)
	{
		// set the bank
		enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, (ECON1_BSEL1|ECON1_BSEL0));
		enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, (address & BANK_MASK)>>5);
		Enc28j60Bank = (address & BANK_MASK);
	}
}

u08 enc28j60Read(u08 address)
{
	// set the bank
	enc28j60SetBank(address);
	// do the read
	return enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address);
}

void enc28j60Write(u08 address, u08 data)
{
	// set the bank
	enc28j60SetBank(address);
	// do the write
	enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, data);
}

void enc28j60PhyWrite(u08 address, u16 data)
{
	// set the PHY register address
	enc28j60Write(MIREGADR, address);
	
	// write the PHY data
	enc28j60Write(MIWRL, data);	
	enc28j60Write(MIWRH, data>>8);

	// wait until the PHY write completes
	while(enc28j60Read(MISTAT) & MISTAT_BUSY);
}

void enc28j60Init(void)
{
    u16 i;
    // Port and SPI init
    spiSetSpeed(5, ETH_CSR); 
	// perform system reset
	enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);
	// check CLKRDY bit to see if reset is complete
	do
	{
		i = enc28j60Read(ESTAT);
	} while((i & 0x08) || (~i & ESTAT_CLKRDY));

	// do bank 0 stuff
	// initialize receive buffer
	// 16-bit transfers, must write low byte first
	// set receive buffer start address
	NextPacketPtr = RXSTART_INIT;
	enc28j60Write(ERXSTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXSTH, RXSTART_INIT>>8);
	// set receive pointer address
	enc28j60Write(ERXRDPTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXRDPTH, RXSTART_INIT>>8);
	// set receive buffer end
	// ERXND defaults to 0x1FFF (end of ram)
	enc28j60Write(ERXNDL, RXSTOP_INIT&0xFF);
	enc28j60Write(ERXNDH, RXSTOP_INIT>>8);
	// set transmit buffer start
	// ETXST defaults to 0x0000 (beginnging of ram)
	enc28j60Write(ETXSTL, TXSTART_INIT&0xFF);
	enc28j60Write(ETXSTH, TXSTART_INIT>>8);

	// do bank 2 stuff
	// enable MAC receive
	enc28j60Write(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);
	// bring MAC out of reset
	enc28j60Write(MACON2, 0x00);
	// enable automatic padding and CRC operations
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, MACON3, MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);

	// set inter-frame gap (non-back-to-back)
	enc28j60Write(MAIPGL, 0x12);
	enc28j60Write(MAIPGH, 0x0C);
	// set inter-frame gap (back-to-back)
	enc28j60Write(MABBIPG, 0x12);
	// Set the maximum packet size which the controller will accept
	enc28j60Write(MAMXFLL, MAX_FRAMELEN&0xFF);	
	enc28j60Write(MAMXFLH, MAX_FRAMELEN>>8);

	// do bank 3 stuff
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, ENC28J60_MAC0);
	enc28j60Write(MAADR4, ENC28J60_MAC1);
	enc28j60Write(MAADR3, ENC28J60_MAC2);
	enc28j60Write(MAADR2, ENC28J60_MAC3);
	enc28j60Write(MAADR1, ENC28J60_MAC4);
	enc28j60Write(MAADR0, ENC28J60_MAC5);

	// no loopback of transmitted frames
	enc28j60PhyWrite(PHCON2, PHCON2_HDLDIS);

	// switch to bank 0
	enc28j60SetBank(ECON1);
	// enable interrutps
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE|EIE_PKTIE);
	// enable packet reception
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);
	
	iprintf("RevID: 0x%x\r\n", enc28j60Read(EREVID));
}

void enc28j60PacketSend(u16 len, u16* packet)
{
	// Set the write pointer to start of transmit buffer area
	enc28j60Write(EWRPTL, TXSTART_INIT);
	enc28j60Write(EWRPTH, TXSTART_INIT>>8);
	// Set the TXND pointer to correspond to the packet size given
	enc28j60Write(ETXNDL, (TXSTART_INIT+len));
	enc28j60Write(ETXNDH, (TXSTART_INIT+len)>>8);

	// write per-packet control byte
	enc28j60WriteOp(ENC28J60_WRITE_BUF_MEM, 0, 0x00);

	// copy the packet into the transmit buffer
	enc28j60WriteBuffer(len, packet);
	
	// send the contents of the transmit buffer onto the network
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS);
}

unsigned int enc28j60PacketReceive(u16 maxlen, u16* packet)
{
	u16 rxstat;
	u16 len;

	// check if a packet has been received and buffered
	if( !(enc28j60Read(EIR) & EIR_PKTIF) )
		return 0;
	
	// Make absolutely certain that any previous packet was discarded	
	//if( WasDiscarded == FALSE)
	//	MACDiscardRx();

	// Set the read pointer to the start of the received packet
	enc28j60Write(ERDPTL, (NextPacketPtr));
	enc28j60Write(ERDPTH, (NextPacketPtr)>>8);
	// read the next packet pointer
	NextPacketPtr  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	NextPacketPtr |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the packet length
	len  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	len |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the receive status
	rxstat  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	rxstat |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;

	// limit retrieve length
	// (we reduce the MAC-reported length by 4 to remove the CRC)
	len = IFMIN(len, maxlen);

	// copy the packet from the receive buffer
	enc28j60ReadBuffer(len, packet);

	// Move the RX read pointer to the start of the next received packet
	// This frees the memory we just read out
	enc28j60Write(ERXRDPTL, (NextPacketPtr));
	enc28j60Write(ERXRDPTH, (NextPacketPtr)>>8);

	// decrement the packet counter indicate we are done with this packet
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);

	return len;
}

void enc28j60RegDump(void)
{

				
	iprintf("RevID: 0x%x\r\n", enc28j60Read(EREVID));

	iprintf("Cntrl: ECON1 ECON2 ESTAT  EIR  EIE\r\n");
	iprintf("%x  ",enc28j60Read(ECON1));
	iprintf("%x  ",enc28j60Read(ECON2));
	iprintf("%x  ",enc28j60Read(ESTAT));
	iprintf("%x  ",enc28j60Read(EIR));
	iprintf("%x  ",enc28j60Read(EIE));
	iprintf("\n");

	iprintf("MAC  : MACON1  MACON2  MACON3  MACON4  MAC-Address\r\n");
	iprintf("%x		");
	iprintf("%x  ",enc28j60Read(MACON1));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(MACON2));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(MACON3));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(MACON4));
	iprintf("%x  ",enc28j60Read(MAADR5));
	iprintf("%x  ",enc28j60Read(MAADR4));
	iprintf("%x  ",enc28j60Read(MAADR3));
	iprintf("%x  ",enc28j60Read(MAADR2));
	iprintf("%x  ",enc28j60Read(MAADR1));
	iprintf("%x  ",enc28j60Read(MAADR0));
	iprintf("\n");

	iprintf("Rx   : ERXST  ERXND  ERXWRPT ERXRDPT ERXFCON EPKTCNT MAMXFL\r\n");
	iprintf("%x		");
	iprintf("%x  ",enc28j60Read(ERXSTH));
	iprintf("%x  ",enc28j60Read(ERXSTL));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(ERXNDH));
	iprintf("%x  ",enc28j60Read(ERXNDL));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(ERXWRPTH));
	iprintf("%x  ",enc28j60Read(ERXWRPTL));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(ERXRDPTH));
	iprintf("%x  ",enc28j60Read(ERXRDPTL));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(ERXFCON));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(EPKTCNT));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(MAMXFLH));
	iprintf("%x  ",enc28j60Read(MAMXFLL));
	iprintf("\n");

	iprintf("Tx   : ETXST  ETXND  MACLCON1 MACLCON2 MAPHSUP\r\n");
	iprintf("%x		");
	iprintf("%x  ",enc28j60Read(ETXSTH));
	iprintf("%x  ",enc28j60Read(ETXSTL));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(ETXNDH));
	iprintf("%x  ",enc28j60Read(ETXNDL));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(MACLCON1));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(MACLCON2));
	iprintf("%x  ");
	iprintf("%x  ",enc28j60Read(MAPHSUP));
	iprintf("\n");

//	delay_us(2500);
}


